﻿using System;
using System.Windows.Forms;
using EZ_B;
using EZ_Builder;
using EZ_Builder.Config.Sub;
using System.Collections.Generic;

namespace Dynamixel {

  public partial class FormDynamixelConfig : Form {

    PluginV1    _cf = new PluginV1();
    DynamixelV1 _v1 = new DynamixelV1();
    DynamixelV2 _v2 = new DynamixelV2();

    public FormDynamixelConfig() {

      InitializeComponent();

      cbChangeIDNew.Items.Clear();
      cbChangeIDOriginal.Items.Clear();
      cbTestServoPort.Items.Clear();

      for (int x = (int)Servo.ServoPortEnum.V0; x < (int)Servo.ServoPortEnum.V99; x++) {

        Servo.ServoPortEnum item = (Servo.ServoPortEnum)x;

        cbChangeIDNew.Items.Add(item);
        cbChangeIDOriginal.Items.Add(item);
        cbTestServoPort.Items.Add(item);
      }

      cbVersion.Items.Clear();
      foreach (FormMain.VersionEnum v in Enum.GetValues(typeof(FormMain.VersionEnum)))
        cbVersion.Items.Add(v);

      cbBaudRates.Items.Clear();
      foreach (DynamixelV1.BAUD_RATES baud in Enum.GetValues(typeof(DynamixelV1.BAUD_RATES)))
        cbBaudRates.Items.Add(baud);

      cbBaudRates.SelectedItem = DynamixelV1.BAUD_RATES.BAUD_57600;
      cbChangeIDNew.SelectedItem = Servo.ServoPortEnum.V2;
      cbChangeIDOriginal.SelectedItem = Servo.ServoPortEnum.V1;
      cbTestServoPort.SelectedItem = Servo.ServoPortEnum.V1;
      cbVersion.SelectedItem = FormMain.VersionEnum.Protocol_1;

      ucTestServoNum.Value = Servo.SERVO_CENTER;
    }

    private void Form_FormClosing(object sender, FormClosingEventArgs e) {

    }

    public void SetConfiguration(PluginV1 cf) {

      _cf = cf;

      ConfigServos configServos = (ConfigServos)cf.GetCustomObjectV2(typeof(ConfigServos));

      flowLayoutPanel1.SuspendLayout();

      foreach (EZ_B.Servo.ServoPortEnum port in Enum.GetValues(typeof(EZ_B.Servo.ServoPortEnum)))
        if (port >= Servo.ServoPortEnum.V0 && port <= Servo.ServoPortEnum.V99) {

          UCServoPort ucServoPort = new UCServoPort();
          ucServoPort.Name = port.ToString();
          ucServoPort.SetPort = port;

          ConfigServosDetail configServoDetail = configServos.GetPort(port);

          if (configServoDetail != null) {

            ucServoPort.SetEnabled = true;
            ucServoPort.SetMaxPosition = configServoDetail.MaxPosition;
            ucServoPort.SetVersion = configServoDetail.Version;
          }

          flowLayoutPanel1.Controls.Add(ucServoPort);
        }

      flowLayoutPanel1.ResumeLayout(true);

      tbBaudRate.Text = cf.STORAGE.GetKey(ConfigTitles.BAUD_RATE, 1000000).ToString();

      tbGlobalMaxServoPosition.Text = cf.STORAGE.GetKey(ConfigTitles.GLOBAL_SERVO_POSITIONS, EZ_B.Servo.SERVO_MAX).ToString();

      var portType = (FormMain.PortTypeEnum)cf.STORAGE.GetKey(ConfigTitles.PORT_TYPE, FormMain.PortTypeEnum.UART);

      if (portType == FormMain.PortTypeEnum.UART)
        rbUARTPort.Checked = true;
      else
        rbDigitalPort.Checked = true;

      ucPortButton1.SetConfig((Digital.DigitalPortEnum)cf.STORAGE.GetKey(ConfigTitles.DIGITAL_PORT, Digital.DigitalPortEnum.D0));
    }

    public PluginV1 GetConfiguration() {

      _cf.STORAGE.AddOrUpdate(ConfigTitles.BAUD_RATE, Convert.ToUInt32(tbBaudRate.Text));
      _cf.STORAGE.AddOrUpdate(ConfigTitles.GLOBAL_SERVO_POSITIONS, Convert.ToInt32(tbGlobalMaxServoPosition.Text));

      if (rbUARTPort.Checked) {

        _cf.STORAGE.AddOrUpdate(ConfigTitles.PORT_TYPE, FormMain.PortTypeEnum.UART);
      } else {

        _cf.STORAGE.AddOrUpdate(ConfigTitles.PORT_TYPE, FormMain.PortTypeEnum.DigitalPort);
        _cf.STORAGE.AddOrUpdate(ConfigTitles.DIGITAL_PORT, ucPortButton1.PortDigital);
      }

      List<ConfigServosDetail> configServosDetails = new List<ConfigServosDetail>();

      foreach (UCServoPort ucServoPort in flowLayoutPanel1.Controls)
        if (ucServoPort.SetEnabled)
          configServosDetails.Add(new ConfigServosDetail() {
            MaxPosition = ucServoPort.SetMaxPosition,
            Port = ucServoPort.SetPort,
            Version = ucServoPort.SetVersion
          });

      ConfigServos configServos = new ConfigServos();
      configServos.Servos = configServosDetails.ToArray();

      _cf.SetCustomObjectV2(configServos);

      return _cf;
    }

    void serialWrite(byte[] data) {

      if (rbDigitalPort.Checked) {

        try {

          var baudRate = (Uart.BAUD_RATE_ENUM)Enum.Parse(typeof(Uart.BAUD_RATE_ENUM), "Baud_" + tbBaudRate.Text);

          EZBManager.EZBs[0].Uart.SendSerial(ucPortButton1.PortDigital, baudRate, data);
        } catch {

          MessageBox.Show("Invalid baud rate for low speed tx port. Valid baud rates are 4800, 9600, 19200, 38400, 57600, 115200");
        }
      } else {

        EZBManager.EZBs[0].Uart.UARTExpansionWrite(1, data);
      }
    }

    private void ucTestServoNum_OnChange(int value) {

      EZBManager.EZBs[0].Servo.SetServoPosition((Servo.ServoPortEnum)cbTestServoPort.SelectedItem, value);
    }

    private void btnChangeID_Click(object sender, EventArgs e) {

      try {

        if (MessageBox.Show("Are you sure?", "Change ID", MessageBoxButtons.YesNo) != System.Windows.Forms.DialogResult.Yes)
          return;

        if ((FormMain.VersionEnum)cbVersion.SelectedItem == FormMain.VersionEnum.Protocol_1)
          serialWrite(_v1.ChangeID((Servo.ServoPortEnum)cbChangeIDOriginal.SelectedItem, (Servo.ServoPortEnum)cbChangeIDNew.SelectedItem));
        else
          serialWrite(_v2.ChangeID((Servo.ServoPortEnum)cbChangeIDOriginal.SelectedItem, (Servo.ServoPortEnum)cbChangeIDNew.SelectedItem));

        MessageBox.Show("Done");
      } catch (Exception ex) {

        MessageBox.Show(ex.Message);
      }
    }

    private void btnLEDOn_Click(object sender, EventArgs e) {

      try {

        if ((FormMain.VersionEnum)cbVersion.SelectedItem == FormMain.VersionEnum.Protocol_1)
          serialWrite(_v1.LED((Servo.ServoPortEnum)cbTestServoPort.SelectedItem, true));
        else
          serialWrite(_v2.LED((Servo.ServoPortEnum)cbTestServoPort.SelectedItem, true));
      } catch (Exception ex) {

        MessageBox.Show(ex.Message);
      }
    }

    private void btnLEDOff_Click(object sender, EventArgs e) {

      try {

        if ((FormMain.VersionEnum)cbVersion.SelectedItem == FormMain.VersionEnum.Protocol_1)
          serialWrite(_v1.LED((Servo.ServoPortEnum)cbTestServoPort.SelectedItem, false));
        else
          serialWrite(_v2.LED((Servo.ServoPortEnum)cbTestServoPort.SelectedItem, false));
      } catch (Exception ex) {

        MessageBox.Show(ex.Message);
      }
    }

    private void button1_Click(object sender, EventArgs e) {

      try {
        if (cbChangeIDNew.SelectedIndex < cbChangeIDNew.Items.Count - 1) {

          cbChangeIDNew.SelectedIndex++;

          cbTestServoPort.SelectedItem = cbChangeIDNew.SelectedItem;
        }
      } catch (Exception ex) {

        MessageBox.Show(ex.Message);
      }
    }

    private void btnDisableStatusCommand_Click(object sender, EventArgs e) {

      try {

        if ((FormMain.VersionEnum)cbVersion.SelectedItem == FormMain.VersionEnum.Protocol_1)
          serialWrite(_v1.DisableStatusPacket((Servo.ServoPortEnum)cbTestServoPort.SelectedItem));
        else
          serialWrite(_v2.DisableStatusPacket((Servo.ServoPortEnum)cbTestServoPort.SelectedItem));
      } catch (Exception ex) {

        MessageBox.Show(ex.Message);
      }
    }

    private void button2_Click(object sender, EventArgs e) {

      try {

        if ((FormMain.VersionEnum)cbVersion.SelectedItem == FormMain.VersionEnum.Protocol_1)
          serialWrite(_v1.DisableStatusPacket());
        else
          serialWrite(_v2.DisableStatusPacket());
      } catch (Exception ex) {

        MessageBox.Show(ex.Message);
      }
    }

    private void button3_Click(object sender, EventArgs e) {

      try {

        if ((FormMain.VersionEnum)cbVersion.SelectedItem == FormMain.VersionEnum.Protocol_1)
          serialWrite(_v1.DisableAllAlarms((Servo.ServoPortEnum)cbTestServoPort.SelectedItem));
        else
          serialWrite(_v2.DisableAllAlarms((Servo.ServoPortEnum)cbTestServoPort.SelectedItem));
      } catch (Exception ex) {

        MessageBox.Show(ex.Message);
      }
    }

    private void btnSave_Click(object sender, EventArgs e) {

      if (!EZ_Builder.Common.IsInteger(tbBaudRate.Text)) {

        MessageBox.Show("Baud rate must be an integer number");

        return;
      }

      if (rbDigitalPort.Checked)
        try {

          Enum.Parse(typeof(Uart.BAUD_RATE_ENUM), "Baud_" + tbBaudRate.Text);

        } catch {

          MessageBox.Show("Valid baudrates for TX low speed serial mode are: 4800, 9600, 19200, 38400, 57600, 115200");

          return;
        }

      DialogResult = System.Windows.Forms.DialogResult.OK;
    }

    private void btnCancel_Click(object sender, EventArgs e) {

      DialogResult = System.Windows.Forms.DialogResult.Cancel;
    }

    private void button4_Click(object sender, EventArgs e) {

      if (!EZ_Builder.Common.IsInteger(tbBaudRate.Text)) {

        MessageBox.Show("Baud rate must be an integer number");

        return;
      }

      if (!EZ_Builder.Common.IsInteger(tbGlobalMaxServoPosition.Text)) {

        MessageBox.Show("Global Max Servo Position must be an integer number");

        return;
      }

      if (!EZBManager.EZBs[0].IsConnected) {

        MessageBox.Show("You must be connected to an EZ-B");

        return;
      }

      if (rbUARTPort.Checked)
        EZBManager.EZBs[0].Uart.UARTExpansionInit(1, Convert.ToUInt32(tbBaudRate.Text));
    }

    private void button5_Click(object sender, EventArgs e) {

      try {

        if (MessageBox.Show(string.Format("Reset servo on port {0} to factory settings?", cbTestServoPort.SelectedItem), "Are you sure?", MessageBoxButtons.YesNo) != DialogResult.Yes)
          return;

        if (rbUARTPort.Checked)
          EZBManager.EZBs[0].Uart.UARTExpansionInit(1, Convert.ToUInt32(tbBaudRate.Text));

        if ((FormMain.VersionEnum)cbVersion.SelectedItem == FormMain.VersionEnum.Protocol_1) {

          serialWrite(_v1.ResetToFactoryDefault((Servo.ServoPortEnum)cbTestServoPort.SelectedItem));

          MessageBox.Show("The baud rate may have changed. Please consult the manual for the servo model", "Done");

        } else {

          MessageBox.Show("This function is not implemented for protocol v2. Please let us know on the ez-robot community forum if you wish to use it");
        }
      } catch (Exception ex) {

        MessageBox.Show(ex.Message);
      }
    }

    private void btnChangeBaud_Click(object sender, EventArgs e) {

      try {

        if (MessageBox.Show(string.Format("Change baud rate on port {0}?", cbTestServoPort.SelectedItem), "Are you sure?", MessageBoxButtons.YesNo) != DialogResult.Yes)
          return;

        if ((FormMain.VersionEnum)cbVersion.SelectedItem == FormMain.VersionEnum.Protocol_1) {

          var baud = (DynamixelV1.BAUD_RATES)cbBaudRates.SelectedItem;
          var port = (Servo.ServoPortEnum)cbTestServoPort.SelectedItem;

          serialWrite(_v1.ChangeBaudRate(port, baud));

          MessageBox.Show("The baud rate has chnged. To communicate with this servo, you must change the baud rate in the first page of this configuration menu", "Done");

        } else {

          MessageBox.Show("This function is not implemented for protocol v2. Please let us know on the ez-robot community forum if you wish to use it");
        }
      } catch (Exception ex) {

        MessageBox.Show(ex.Message);
      }
    }
  }
}
